-------------------------------------------------------------------------
--
--  sram_access.vhd
--
--	Author 	: Rene Bllhoff
--  Create 	: Sep. 2006
--
--  Purpose : 
--
--	Komponente zur Erzeugung eines Dual-Port RAMs aus dem SRAM
--  - Statemachine ist absoluter Master im System
--    - stellt einen Lese-Port (32-Bit) zur Verfgung
--    - stellt einen Lese/Schreib Port (8-Bit) zur Verfgung
--  - Lese/Schreib-Zugriffe mssen ber ri_trg signalisiert werden, nachdem
--    ri_req angezeigt hat das ein Zugriff mglich ist
--  - maximaler Datendurchsatz (rein rechnerisch) : 6.25 MByte/sek.
--
--  (c)2005-2006 by Rene Bllhoff
-------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sram_access is
		port (clk 			: in		std_logic;
					res 			: in 		std_logic;

					-- sram seite (fr 2 srams)

					sri_adr		: out 	std_logic_vector (17 downto 0);
					sri_data1	: inout std_logic_vector (15 downto 0);
					sri_data2	: inout std_logic_vector (15 downto 0);
					sri_cs1		: out		std_logic;
					sri_cs2		: out		std_logic;
					sri_ub1		: out		std_logic;
					sri_lb1		: out		std_logic;
					sri_ub2		: out		std_logic;
					sri_lb2		: out		std_logic;
					sri_oe		: out		std_logic;
					sri_we		: out		std_logic;

					-- fpga seite 1

					ri_adr		: in  	std_logic_vector (19 downto 0);
					ri_din		: in    std_logic_vector ( 7 downto 0);
					ri_dout		: out 	std_logic_vector ( 7 downto 0);
					ri_rw			: in		std_logic;
					ri_trg		: in		std_logic;
					ri_rdy		: out		std_logic;
					ri_req		: out		std_logic;

					-- fpga seite 2

					ri_radr  	: in    std_logic_vector (17 downto 0);
					ri_rdata	: out   std_logic_vector (31 downto 0);
					ri_rda		: out 	std_logic

				 );
end sram_access;

architecture Behavioral of sram_access is


	type ram_states is (rs_load_ri2_1, 
											rs_load_ri2_2, 
											rs_load_ri2_3, 
											rs_test_ri1, 
											
											rs_ri1_read, 
											rs_ri1_read_1, 
											rs_ri1_read_2, 
											rs_ri1_read_3, 

											rs_ri1_write, 
											rs_ri1_write_1, 
											rs_ri1_write_2, 
											rs_ri1_write_3, 

											rs_ri1_nop, 
											rs_ri1_nop_1,
											rs_ri1_nop_2, 
											rs_ri1_nop_3

											);

	signal ram_state : ram_states;
	signal ram_data1 : std_logic_vector ( 7 downto 0);
	signal ram_data2 : std_logic_vector (31 downto 0);

begin

	process (clk, res, 
	         sri_data1, sri_data2,
					 ram_data1, ram_data2,
	         ram_state, ri_adr, ri_din, ri_rw, ri_trg)
	begin

		ri_dout 	<= ram_data1;
		ri_rdata 	<= ram_data2;		

		if res = '1' then

			ram_state <= rs_ri1_nop_3;			

			sri_data1	<= (others => 'Z');
			sri_data2	<= (others => 'Z');
			sri_cs1 	<= '1';
			sri_cs2		<= '1';
			sri_ub1  	<= '1';
			sri_lb1  	<= '1';
			sri_ub2 	<= '1';
			sri_lb2  	<= '1';
			sri_we 	 	<= '1';
			sri_oe 		<= '1';
			ri_req		<= '0';

			
			ram_data1 <= (others => '0');
			ram_data2 <= (others => '0');
      ri_rdy 		<= '0';
			ri_rda 		<= '0';

		elsif clk'event and clk = '1' then
	


			case ram_state is
				when rs_load_ri2_1  => 	
																sri_data1 	<= (others => 'Z');
																sri_data2 	<= (others => 'Z');
																sri_adr			<= ri_radr;
																sri_cs1 		<= '1';
																sri_cs2 		<= '1';
																sri_ub1  		<= '0';
																sri_lb1  		<= '0';
																sri_ub2 		<= '0';
																sri_lb2  		<= '0';
																sri_oe  		<= '1';
																sri_we			<= '1';
																ri_rdy 			<= '0';
																ri_req			<= '1';
																ri_rda 			<= '0';
																ram_state		<= rs_load_ri2_2;

				when rs_load_ri2_2  => 
																sri_cs1 		<= '0';
																sri_cs2 		<= '0';
																sri_ub1  		<= '0';
																sri_lb1  		<= '0';
																sri_lb1  		<= '0';
																sri_ub2 		<= '0';
																sri_oe  		<= '0';
																sri_we			<= '1';
																ri_req			<= '1';
																ri_rda 			<= '0';
																ram_state		<= rs_load_ri2_3;

				when rs_load_ri2_3  => 	
																ram_data2 (31 downto 16) <= sri_data2;
																ram_data2 (15 downto  0) <= sri_data1;
																sri_cs1 		<= '1';
																sri_cs2 		<= '1';
																sri_ub1  		<= '1';
																sri_lb1  		<= '1';
																sri_ub2  		<= '1';
																sri_lb2  		<= '1';
																sri_oe  		<= '1';
																sri_oe  		<= '1';
																sri_we			<= '1';
																ri_req			<= '0';
																ri_rda 			<= '0';
																ram_state		<= rs_test_ri1;

				when rs_test_ri1		=>  
																ri_rda 			<= '1';
																sri_adr 		<= 			ri_adr (19 downto 2);
																if ri_trg = '1' then
																	if ri_rw = '1' then
																		sri_data1 	<= (others => 'Z');
																		sri_data2 	<= (others => 'Z');

																		sri_oe  		<= '1';
																		sri_we			<= '1';
																		ram_state		<= rs_ri1_read;

																	else
																		sri_data1 	<= (others => 'Z');
																		sri_data2 	<= (others => 'Z');

																		sri_lb1  		<=     ri_adr ( 0) or      ri_adr ( 1);
																		sri_ub1 		<= not ri_adr ( 0) or      ri_adr ( 1);
																		sri_lb2  		<=     ri_adr ( 0) or  not ri_adr ( 1);
																		sri_ub2  		<= not ri_adr ( 0) or  not ri_adr ( 1);

																		sri_oe  		<= '1';
																		sri_we			<= '0';
																		ram_state		<= rs_ri1_write;
																	end if;
																else
																	sri_cs1 		<= '1';
																	sri_cs2 		<= '1';
																	sri_ub1  		<= '1';
																	sri_lb1  		<= '1';
																	sri_ub2  		<= '1';
																	sri_lb2  		<= '1';
																	sri_oe  		<= '1';
																	sri_we			<= '1';
																	ram_state		<= rs_ri1_nop;
																end if;


				-- ram - read
				when rs_ri1_read  		=>	ri_rda 			<= '0';
																	sri_cs1 		<=     ri_adr ( 1);
																	sri_cs2 		<= not ri_adr ( 1);
																	sri_lb1  		<=     ri_adr ( 0) or      ri_adr ( 1);
																	sri_ub1 		<= not ri_adr ( 0) or      ri_adr ( 1);
																	sri_lb2  		<=     ri_adr ( 0) or  not ri_adr ( 1);
																	sri_ub2  		<= not ri_adr ( 0) or  not ri_adr ( 1);
																	
																	sri_data1 	<= (others => 'Z');
																	sri_data2 	<= (others => 'Z');
																	sri_oe  		<= '0';
																	sri_we  		<= '1';
																	ram_state		<= rs_ri1_read_1;

																	

				when rs_ri1_read_1		=>	ri_rda 			<= '0';
																	ram_state		<= rs_ri1_read_2;

				when rs_ri1_read_2		=>	
																	case ri_adr (1 downto 0) is
																		when "00" 	=> ram_data1 <= sri_data1 ( 7 downto 0);
																		when "01" 	=> ram_data1 <= sri_data1 (15 downto 8);
																		when "10" 	=> ram_data1 <= sri_data2 ( 7 downto 0);
																		when "11" 	=> ram_data1 <= sri_data2 (15 downto 8);
																	  when others => ram_data1 <= (others => '0');
																  end case;		
																	ram_state		<= rs_ri1_read_3;

				when rs_ri1_read_3		=>	
																	sri_cs1 		<= '1';
																	sri_cs2 		<= '1';
																	sri_ub1  		<= '1';
																	sri_lb1  		<= '1';
																	sri_ub2  		<= '1';
																	sri_lb2  		<= '1';
																	sri_oe  		<= '1';
																	sri_we			<= '1';
																	ri_rdy 			<= '0';
																	ram_state		<= rs_load_ri2_1;

				-- ram - write
				when rs_ri1_write			=>	
																	ri_rda 			<= '0';

																	sri_cs1 		<=     ri_adr ( 1);
																	sri_cs2 		<= not ri_adr ( 1);
																	sri_lb1  		<=     ri_adr ( 0) or      ri_adr ( 1);
																	sri_ub1 		<= not ri_adr ( 0) or      ri_adr ( 1);
																	sri_lb2  		<=     ri_adr ( 0) or  not ri_adr ( 1);
																	sri_ub2  		<= not ri_adr ( 0) or  not ri_adr ( 1);

																	case ri_adr (1 downto 0) is
																	  when "00" => 	sri_data1 ( 7 downto  0) 	<= ri_din;
																		when "01" => 	sri_data1 (15 downto  8) 	<= ri_din; 
																		when "10" => 	sri_data2 ( 7 downto  0) 	<= ri_din;
																		when "11" => 	sri_data2 (15 downto  8) 	<= ri_din;
																		when others => 
																									sri_ub1  		<= '1';
																									sri_lb1  		<= '1';
																									sri_ub2  		<= '1';
																									sri_lb2  		<= '1';
																									sri_data1	 	<= (others => 'Z');
																									sri_data2 	<= (others => 'Z');
																	end case;

																	sri_we			<= '0';
																	sri_oe  		<= '1';
																	ram_state		<= rs_ri1_write_1;


				when rs_ri1_write_1		=>	
																	ri_rda 			<= '0';

																	sri_we			<= '0';
																	sri_oe  		<= '1';
																	ram_state		<= rs_ri1_write_2;

				when rs_ri1_write_2		=>	sri_we			<= '1';
																	sri_ub1  		<= '1';
																	sri_lb1  		<= '1';
																	sri_ub2  		<= '1';
																	sri_lb2  		<= '1';
																	sri_oe  		<= '1';
																	sri_data1	 	<= (others => 'Z');
																	sri_data2 	<= (others => 'Z');
																	ram_state		<= rs_ri1_write_3;

				when rs_ri1_write_3		=>	
																	sri_data1	 	<= (others => 'Z');
																	sri_data2 	<= (others => 'Z');
																	sri_cs1 		<= '1';
																	sri_cs2 		<= '1';
																	sri_ub1  		<= '1';
																	sri_lb1  		<= '1';
																	sri_ub2  		<= '1';
																	sri_lb2  		<= '1';
																	sri_oe  		<= '1';
																	sri_we			<= '1';
																	ri_rdy 			<= '1';
																	ram_state		<= rs_load_ri2_1;
																
																
				-- ram - nop
				when rs_ri1_nop  			=>	ri_rda 			<= '0';
																	ram_state		<= rs_ri1_nop_1;
				when rs_ri1_nop_1			=>	ram_state		<= rs_ri1_nop_2;
				when rs_ri1_nop_2			=>	ram_state		<= rs_ri1_nop_3;
				when rs_ri1_nop_3			=>	ram_state		<= rs_load_ri2_1;

				when others 					=>	ram_state		<= rs_load_ri2_1;

			end case;

		end if;


	end process;

end Behavioral;
